#include "utils.h"
#include <sstream>
using namespace std;

namespace TestStream
{

#define TESTNAME "TestStream"

stringstream stream;

class CScriptOutStream
{
public:
	CScriptOutStream();
	~CScriptOutStream();

	CScriptOutStream &operator=(const CScriptOutStream&);

	stringstream *s;
};

CScriptOutStream &operator<<(CScriptOutStream &s, const string &other)
{
//	printf("(%X) << \"%s\"\n", &s, other.c_str());

	*s.s << other;
	return s;
}

CScriptOutStream::CScriptOutStream()
{
//	printf("new (%X)\n", this);

	s = &stream;
}

CScriptOutStream::~CScriptOutStream()
{
//	printf("del (%X)\n", this);
}

CScriptOutStream &CScriptOutStream::operator=(const CScriptOutStream &other)
{
//	printf("(%X) = (%X)\n", this, &other);

	s = other.s;

	return *this;
}

void CScriptOutStream_Construct(CScriptOutStream *o)
{
	new(o) CScriptOutStream;
}

void CScriptOutStream_Destruct(CScriptOutStream *o)
{
	o->~CScriptOutStream();
}

static const char *script1 =
"void Test()                   \n"
"{                             \n"
"  ostream s;                  \n"
"  s << \"a\" << \"b\" << \"c\"; \n"
"}                             \n";

bool Test()
{
	bool fail = false;
	int r;

 	asIScriptEngine *engine = asCreateScriptEngine(ANGELSCRIPT_VERSION);

	RegisterScriptString(engine);

	engine->RegisterObjectType("ostream", sizeof(CScriptOutStream), asOBJ_CLASS_CDA);
	engine->RegisterObjectBehaviour("ostream", asBEHAVE_CONSTRUCT, "void f()", asFUNCTION(CScriptOutStream_Construct), asCALL_CDECL_OBJLAST);
	engine->RegisterObjectBehaviour("ostream", asBEHAVE_DESTRUCT, "void f()", asFUNCTION(CScriptOutStream_Destruct), asCALL_CDECL_OBJLAST);
	engine->RegisterObjectBehaviour("ostream", asBEHAVE_ASSIGNMENT, "ostream &f(const ostream &in)", asMETHOD(CScriptOutStream, operator=), asCALL_THISCALL);
	engine->RegisterGlobalBehaviour(asBEHAVE_BIT_SLL, "ostream &f(ostream &in, string &in)", asFUNCTIONPR(operator<<, (CScriptOutStream &s, const string &other), CScriptOutStream &), asCALL_CDECL);

	COutStream out;
	engine->AddScriptSection(0, TESTNAME, script1, strlen(script1), 0, false);
	r = engine->Build(0, &out);
	if( r < 0 )
	{
		fail = true;
		printf("%s: Failed to compile the script\n", TESTNAME);
	}

	asIScriptContext *ctx;
	r = engine->ExecuteString(0, "Test()", 0, &ctx);
	if( r != asEXECUTION_FINISHED )
	{
		if( r == asEXECUTION_EXCEPTION )
			PrintException(ctx);

		printf("%s: Failed to execute script\n", TESTNAME);
		fail = true;
	}
	if( ctx ) ctx->Release();

	if( stream.str() != "abc" )
	{
		printf("%s: Failed to create the correct stream\n", TESTNAME);
		fail = true;
	}

	engine->Release();

	// Success
	return fail;
}

} // namespace

